@@ -1,17 +1,3 @@
-#' Create NULL columns and the line 0
-#'
-#' @param data [data.frame] 
-#' A data frame resulting from the 'step2' process of the `arrange_data` function. 
-#'
-#' @returns data frame:
-#'  \itemize{
-#'    \item{
-#'      \code{data}: step2 + row[0] for initial value + null cols 
-#'      [Reward, gamma, R_utility, ...].}
-#'  }
-#'   
-#' @noRd
-#' 
 add_NA <- function(data){
   
   # 生成一个与输入数据集相同的单行数据集. 用于存放初始值
@@ -20,12 +6,9 @@ add_NA <- function(data){
   # 在第一行插入一个空行
   data <- rbind(empty_row, data)
   
-  # 添加空列 action_c 相关
-  data$L_freq <- NA
-  data$R_freq <- NA
+  data$Rob_Choose <- NA
   
-  data$L_pick <- NA
-  data$R_pick <- NA
+  data$ACC <- NA
   
   data$L_value <- NA
   data$R_value <- NA
@@ -36,9 +19,18 @@ add_NA <- function(data){
   data$L_prob <- NA
   data$R_prob <- NA
   
-  data$Try <- NA
+  data$L_dir <- NA
+  data$R_dir <- NA
   
-  data$Rob_Choose <- NA
+  data$L_logl <- NA
+  data$R_logl <- NA
+  
+  # 添加空列 action_c 相关
+  data$L_freq <- NA
+  data$R_freq <- NA
+  
+  data$L_pick <- NA
+  data$R_pick <- NA
   
   # 添加空列 update_v 相关
   data$Occurrence <- NA
@@ -51,5 +43,7 @@ add_NA <- function(data){
   data$eta <- NA
   data$V_update <- NA
   
+  data$Try <- NA
+  
   return(data)
 }
